@@ -17,6 +17,10 @@ from functools import partial, wraps
 
 
 class CornFlow(object):
+    """
+    Base class to access cornflow-server
+    """
+
     def __init__(self, url, token=None):
         self.url = url
         self.token = token
@@ -46,6 +50,15 @@ class CornFlow(object):
     #     return partial(expect_status, status=200)
 
     def api_for_id(self, api, id, method, post_url="", **kwargs):
+        """
+        :param api: the resource in the server
+        :param id: the id of the particular object
+        :param method: HTTP method to apply
+        :param post_url: optional action to apply
+        :param kwargs: other arguments to requests.request
+
+        :return: requests.request
+        """
         if post_url and post_url[-1] != "/":
             post_url += "/"
         url = urljoin(urljoin(self.url, api) + "/", str(id) + "/" + post_url)
@@ -58,24 +71,39 @@ class CornFlow(object):
 
     @ask_token
     def get_api_for_id(self, api, id, post_url="", **kwargs):
+        """
+        api_for_id with a GET request
+        """
         return self.api_for_id(
             api=api, id=id, post_url=post_url, **kwargs, method="get"
         )
 
     @ask_token
     def delete_api_for_id(self, api, id, **kwargs):
+        """
+        api_for_id with a DELETE request
+        """
         return self.api_for_id(api=api, id=id, **kwargs, method="delete")
 
     @ask_token
     def put_api_for_id(self, api, id, payload, **kwargs):
+        """
+        api_for_id with a PUT request
+        """
         return self.api_for_id(api=api, id=id, json=payload, method="put", **kwargs)
 
     @ask_token
     def patch_api_for_id(self, api, id, payload, **kwargs):
+        """
+        api_for_id with a PATCH request
+        """
         return self.api_for_id(api=api, id=id, json=payload, method="patch", **kwargs)
 
     @ask_token
     def post_api_for_id(self, api, id, **kwargs):
+        """
+        api_for_id with a POST request
+        """
         return self.api_for_id(api=api, id=id, method="post", **kwargs)
 
     @ask_token
@@ -88,6 +116,15 @@ class CornFlow(object):
 
     @log_call
     def sign_up(self, username, email, pwd):
+        """
+        Sign-up to the server. Creates a new user or returns error.
+
+        :param str username: username
+        :param str email: email
+        :param str pwd: password
+
+        :return: a dictionary with a token inside
+        """
         return requests.post(
             urljoin(self.url, "signup/"),
             json={"username": username, "password": pwd, "email": email},
@@ -95,6 +132,9 @@ class CornFlow(object):
 
     @log_call
     def is_alive(self):
+        """
+        Asks the server if it's alive
+        """
         response = requests.get(urljoin(self.url, "health/"))
         if response.status_code == 200:
             return response.json()
@@ -105,6 +145,14 @@ class CornFlow(object):
         )
 
     def login(self, username, pwd):
+        """
+        Log-in to the server.
+
+        :param str username: username
+        :param str pwd: password
+
+        :return: a dictionary with a token inside
+        """
         response = requests.post(
             urljoin(self.url, "login/"), json={"username": username, "password": pwd}
         )
@@ -125,6 +173,14 @@ class CornFlow(object):
     def create_instance(
         self, data, name=None, description="", schema="solve_model_dag"
     ):
+        """
+        Uploads an instance to the server
+
+        :param dict data: data according to json-schema for the problem
+        :param str name: name for instance
+        :param str description: description of the instance
+        :param str schema: name of problem to solve
+        """
         if name is None:
             try:
                 name = data["parameters"]["name"]
@@ -143,6 +199,16 @@ class CornFlow(object):
     def create_case(
         self, name, schema, data=None, parent_id=None, description="", solution=None
     ):
+        """
+        Uploads a case to the server
+
+        :param dict data: data according to json-schema for the problem
+        :param str name: name for case
+        :param str description: description of case
+        :param str schema: name of problem of case
+        :param str parent_id: id of the parent directory in the tree structure
+        :param dict solution: optional solution data to store inside case
+        """
         payload = dict(
             name=name,
             description=description,
@@ -168,6 +234,13 @@ class CornFlow(object):
     @ask_token
     @log_call
     def create_instance_file(self, filename, name, description="", minimize=True):
+        """
+        Uploads a file to the server to be parsed into an instance
+
+        :param str filename: path to filename to upload
+        :param str name: name for instance
+        :param str description: description of the instance
+        """
         with open(filename, "rb") as file:
             response = self.create_api(
                 "instancefile/",
@@ -193,6 +266,15 @@ class CornFlow(object):
         description="",
         schema="solve_model_dag",
     ):
+        """
+        Creates an execution from a (previously) uploaded instance
+
+        :param str instance_id: id for the instance
+        :param str name: name for the execution
+        :param str description: description of the execution
+        :param dict config: execution configuration
+        :param str schema: name of the problem to solve
+        """
         payload = dict(
             config=config,
             instance_id=instance_id,
@@ -211,15 +293,26 @@ class CornFlow(object):
 
     @ask_token
     def get_data(self, execution_id):
+        """
+        Downloads the data from an execution
+
+        :param str execution_id: id for the execution
+        """
         response = self.get_api_for_id(api="dag/", id=execution_id)
         return response.json()
 
     @ask_token
     def write_solution(self, execution_id, **kwargs):
+        """
+        Edits an execution
+
+        :param str execution_id: id for the execution
+        :param kwargs: optional data to edit
+        """
         response = self.put_api_for_id("dag/", id=execution_id, payload=kwargs)
-        if response.status_code != 201:
+        if response.status_code != 200:
             raise CornFlowApiError(
-                "Expected a code 201, got a {} error instead: {}".format(
+                "Expected a code 200, got a {} error instead: {}".format(
                     response.status_code, response.text
                 )
             )
@@ -228,6 +321,11 @@ class CornFlow(object):
     @log_call
     @ask_token
     def stop_execution(self, execution_id):
+        """
+        Interrupts an ongoing execution
+
+        :param str execution_id: id for the execution
+        """
         response = self.post_api_for_id(api="execution/", id=execution_id)
         if response.status_code != 200:
             raise CornFlowApiError(
@@ -239,6 +337,16 @@ class CornFlow(object):
 
     @ask_token
     def manual_execution(self, instance_id, config, name, **kwargs):
+        """
+        Uploads an execution from solution data
+
+        :param str instance_id: id for the instance
+        :param str name: name for execution
+        :param str description: description of the execution
+        :param dict config: execution configuration
+        :param str schema: name of the problem to solve
+        :param kwargs: contents of the solution (inside data)
+        """
         payload = dict(config=config, instance_id=instance_id, name=name, **kwargs)
         response = self.create_api("dag/", json=payload)
         if response.status_code != 201:
@@ -252,12 +360,22 @@ class CornFlow(object):
     @log_call
     @ask_token
     def get_results(self, execution_id):
+        """
+        Downloads results from an execution
+
+        :param str execution_id: id for the execution
+        """
         response = self.get_api_for_id(api="execution/", id=execution_id)
         return response.json()
 
     @log_call
     @ask_token
     def get_status(self, execution_id):
+        """
+        Downloads the current status of an execution
+
+        :param str execution_id: id for the execution
+        """
         response = self.get_api_for_id(
             api="execution/", id=execution_id, post_url="status"
         )
@@ -266,6 +384,11 @@ class CornFlow(object):
     @log_call
     @ask_token
     def get_log(self, execution_id):
+        """
+        Downloads the log for an execution
+
+        :param str execution_id: id for the execution
+        """
         response = self.get_api_for_id(
             api="execution/", id=execution_id, post_url="log"
         )
@@ -274,6 +397,11 @@ class CornFlow(object):
     @log_call
     @ask_token
     def get_solution(self, execution_id):
+        """
+        Downloads the solution data for an execution
+
+        :param str execution_id: id for the execution
+        """
         response = self.get_api_for_id(
             api="execution/", id=execution_id, post_url="data"
         )
@@ -282,6 +410,11 @@ class CornFlow(object):
     @log_call
     @ask_token
     def get_all_instances(self, params=None):
+        """
+        Downloads all the user's instances
+
+        :param dict params: optional filters
+        """
         response = requests.get(
             urljoin(self.url, "instance/"),
             headers={"Authorization": "access_token " + self.token},
@@ -293,6 +426,11 @@ class CornFlow(object):
     @log_call
     @ask_token
     def get_all_cases(self, params=None):
+        """
+        Downloads all the user's cases
+
+        :param dict params: optional filters
+        """
         response = requests.get(
             urljoin(self.url, "case/"),
             headers={"Authorization": "access_token " + self.token},
@@ -304,6 +442,11 @@ class CornFlow(object):
     @log_call
     @ask_token
     def get_all_executions(self, params=None):
+        """
+        Downloads all the user's executions
+
+        :param dict params: optional filters
+        """
         response = requests.get(
             urljoin(self.url, "execution/"),
             headers={"Authorization": "access_token " + self.token},
@@ -315,6 +458,9 @@ class CornFlow(object):
     @log_call
     @ask_token
     def get_all_users(self):
+        """
+        Downloads all the users in the server
+        """
         return requests.get(
             urljoin(self.url, "user/"),
             headers={"Authorization": "access_token " + self.token},
@@ -324,46 +470,88 @@ class CornFlow(object):
     @log_call
     @ask_token
     def get_one_user(self, user_id):
+        """
+        Downloads all information on a user
+
+        :param str user_id: user id
+        """
         return self.get_api_for_id(api="user", id=user_id)
 
     @log_call
     @ask_token
     def get_one_instance(self, reference_id):
+        """
+        Downloads header of an instance
+
+        :param str reference_id: id for the instance
+        """
         response = self.get_api_for_id(api="instance", id=reference_id)
         return response.json()
 
     @log_call
     @ask_token
     def get_one_case(self, reference_id):
+        """
+        Downloads header of a case
+
+        :param str reference_id: id for the case
+        """
         response = self.get_api_for_id(api="case", id=reference_id)
         return response.json()
 
     @log_call
     @ask_token
     def delete_one_case(self, reference_id):
+        """
+        Deletes a case
+
+        :param str reference_id: id for the case
+        """
         response = self.delete_api_for_id(api="case", id=reference_id)
         return response.json()
 
     @log_call
     @ask_token
     def put_one_case(self, reference_id, payload):
+        """
+        Edits a case
+
+        :param str reference_id: id for the case
+        :param dict payload: data to edit
+        """
         response = self.put_api_for_id(api="case", id=reference_id, payload=payload)
         return response.json()
 
     @log_call
     @ask_token
     def patch_one_case(self, reference_id, payload):
+        """
+        Patch a case
+
+        :param str reference_id: id for the case
+        :param dict payload: data to edit
+        """
         response = self.patch_api_for_id(api="case", id=reference_id, payload=payload)
         return response.json()
 
     @log_call
     @ask_token
     def delete_one_instance(self, reference_id):
+        """
+        Delete an instance
+
+        :param str reference_id: id for the instance
+        """
         response = self.delete_api_for_id(api="instance", id=reference_id)
         return response
 
     @ask_token
     def get_schema(self, dag_name):
+        """
+        Downloads schemas for a problem
+
+        :param str dag_name: id for the problem
+        """
         response = self.get_api_for_id(api="schema", id=dag_name)
         return response.json()
 
